@@ -2,8 +2,9 @@
 #
 # read.cross.mq.R
 #
-# copyright (c) 2014, INRA (author: Timothée Flutre)
-# last modified June, 2014
+# copyright (c) 2014, INRA (author: Timothee Flutre)
+#                     (Some revisions by Karl Broman)
+# last modified Dec, 2018
 # first written May, 2014
 #
 # This program is free software; you can redistribute it and/or
@@ -35,7 +36,7 @@
 # positions, and a "qua" file containing the phenotypes.
 #
 # File formats are described in the MapQTL manual available online at
-# http://www.kyazma.nl/docs/MQ6Manual.pdf
+# http://www.kyazma.nl/docs/MQ7Manual.pdf
 # For the loc-file, each marker should be on a single line.
 # Only 4-way crosses are supported ("CP" type in MapQTL/JoinMap).
 #
@@ -128,7 +129,7 @@ function(locfile){
     classif <- NULL
     genotypes <- NULL
 
-    lines <- readLines(locfile)
+    lines <- readLines(locfile, warn=FALSE)
 
     # drop comments
     lines <- vapply(strsplit(lines, ";"), "[", "", 1)
@@ -180,12 +181,17 @@ function(locfile){
     lines <- lines[-todrop]
 
     spl <- strsplit(lines, "\\s+")
+    if(length(spl) != nb.loci)
+      stop("nloc=", nb.loci, " but genotypes are found at ", length(spl), " markers")
     spl.lengths <- vapply(spl, length, 1)
     if(any(spl.lengths > nb.inds+4))
         stop("lines should have no more than ", nb.inds+4, " columns\n",
              "Problems in lines", seq(along=spl.lengths)[spl.lengths > nb.inds+4])
 
-    rownames(genotypes) <- vapply(spl, "[", "", 1)
+    rn <- vapply(spl, "[", "", 1)
+    if(nrow(genotypes) != length(rn))
+        stop(paste0("nloc = ", nrow(genotypes), ", but .loc file contains ", length(rn), " genotype rows"))
+    rownames(genotypes) <- rn
 
     if(length(spl) > nb.loci + 1){
         msg <- paste("there seems to be more loci (", locus.id-1,
@@ -193,8 +199,10 @@ function(locfile){
         stop(msg, call.=FALSE)
     }
 
+    nb.fields <- rep(NA, length(lines))
     for(line.id in 1:length(lines)){
         tokens <- spl[[line.id]]
+        nb.fields[line.id] <- length(tokens)
 
         if(length(tokens) > nb.inds + 1){
             for(i in 2:(length(tokens)-nb.inds)){
@@ -208,6 +216,8 @@ function(locfile){
         }
         genotypes[line.id,] <- tokens[(length(tokens)-nb.inds+1):length(tokens)]
     }
+    if(length(unique(nb.fields)) > 1)
+        stop("some markers have more fields than others")
 
     genotypes <- t(genotypes) # individuals in rows, markers in columns
 
@@ -237,7 +247,7 @@ function(locfile){
             stop(msg, call.=FALSE)
         }
         for(locus.id in 1:nb.loci){
-            if(seg[locus.id] == "<abxcd>"){
+            if(seg[locus.id] %in% new.seg.types){
                 next
             } else if(seg[locus.id] == "<abxac>"){
                 seg[locus.id] <- "<efxeg>"
@@ -587,7 +597,7 @@ read.cross.mq.map <-
 function(mapfile){
 
     # read all lines
-    lines <- readLines(mapfile)
+    lines <- readLines(mapfile, warn=FALSE)
 
     # remove comments
     lines <- vapply(strsplit(lines, ";"), "[", "", 1)
@@ -646,7 +656,7 @@ function(quafile){
     miss <- NULL
     phenotypes <- NULL
 
-    lines <- readLines(quafile)
+    lines <- readLines(quafile, warn=FALSE)
 
     # remove comments
     lines <- vapply(strsplit(lines, ";"), "[", "", 1)
